import aw_client
from datetime import datetime, timedelta
import socket
import netifaces
import json
import requests
import sys


def initialize_aw_client(client_name="my-client"):
    """Initialize the ActivityWatch client."""
    try:
        client = aw_client.ActivityWatchClient(client_name)
        return client
    except Exception as e:
        print(f"Failed to initialize ActivityWatch client: {e}")
        sys.exit(1)


def get_time_range(minutes=60):
    """Get the current time and the time 'minutes' ago, truncated to the hour."""
    now = datetime.now()
    time_ago = now - timedelta(minutes=minutes)
    # Truncate to the start of the hour
    time_ago_truncated = datetime(time_ago.year, time_ago.month, time_ago.day, time_ago.hour)
    return time_ago_truncated


def filter_buckets(buckets):
    """Filter buckets related to windows, afk, and web activities."""
    window_buckets = [bucket for bucket in buckets.values() if 'window' in bucket.get('client', '')]
    afk_buckets = [bucket for bucket in buckets.values() if 'afk' in bucket.get('id', '').lower()]
    web_buckets = [bucket for bucket in buckets.values() if 'web' in bucket.get('id', '').lower()]
    return window_buckets, afk_buckets, web_buckets


def get_bucket_events(client, buckets, start_time):
    """Retrieve events for each bucket starting from 'start_time'."""
    bucket_event_map = {}
    all_events = []
    for bucket in buckets:
        try:
            events = client.get_events(bucket['id'], start=start_time)
            bucket_event_map[bucket['id']] = events
            all_events.extend(events)
        except Exception as e:
            print(f"Failed to get events for bucket {bucket['id']}: {e}")
    return bucket_event_map, all_events


def format_time(timestamp):
    """Convert UTC timestamp to local time."""
    try:
        if isinstance(timestamp, datetime):
            utc_time = timestamp
        else:
            utc_time = datetime.fromisoformat(timestamp.replace("Z", "+00:00"))
        local_time = utc_time.astimezone()
        return local_time
    except Exception as e:
        print(f"Error formatting time: {e}")
        return None


def get_machine_info():
    """Retrieve the machine's hostname and non-loopback IP address."""
    try:
        hostname = socket.gethostname()
        ip_address = 'Unable to get IP address'
        for interface in netifaces.interfaces():
            addresses = netifaces.ifaddresses(interface)
            if netifaces.AF_INET in addresses:
                for link in addresses[netifaces.AF_INET]:
                    ip = link.get('addr')
                    if ip and not ip.startswith("127."):
                        ip_address = ip
                        return hostname, ip_address
        return hostname, ip_address
    except Exception as e:
        print(f"Error retrieving machine info: {e}")
        return 'Unknown Hostname', 'Unknown IP'


def process_events(bucket_event_map, window_buckets, afk_buckets, web_buckets):
    """Process events from different buckets and categorize them."""
    app_activity_periods = {}
    afk_periods = []
    web_activity = []

    web_bucket_ids = {bucket['id'] for bucket in web_buckets}
    afk_bucket_ids = {bucket['id'] for bucket in afk_buckets}
    window_bucket_ids = {bucket['id'] for bucket in window_buckets}

    for bucket_id, events in bucket_event_map.items():
        if bucket_id in web_bucket_ids:
            for event in events:
                url = event['data'].get('url', 'Unknown URL')
                title = event['data'].get('title', 'Unknown Title')
                start_time = format_time(event['timestamp'])
                duration = event.get('duration')
                if isinstance(duration, timedelta) and start_time:
                    duration_minutes = duration.total_seconds() / 60
                    start_time_truncated = start_time.replace(minute=0, second=0, microsecond=0)
                    web_activity.append({
                        'url': url,
                        'title': title,
                        'start_time': start_time_truncated.strftime('%Y%m%d %H:%M:%S'),
                        'duration': duration_minutes
                    })

        elif bucket_id in afk_bucket_ids:
            for event in events:
                status = event['data'].get('status', 'Unknown Status')
                start_time = format_time(event['timestamp'])
                duration = event.get('duration')
                if isinstance(duration, timedelta) and start_time:
                    duration_minutes = duration.total_seconds() / 60
                    start_time_truncated = start_time.replace(minute=0, second=0, microsecond=0)
                    afk_periods.append({
                        'status': status,
                        'start_time': start_time_truncated.strftime('%Y%m%d %H:%M:%S'),
                        'duration': duration_minutes
                    })

        elif bucket_id in window_bucket_ids:
            for event in events:
                app = event['data'].get('app')
                title = event['data'].get('title', 'Unknown Title')
                if app:
                    start_time = format_time(event['timestamp'])
                    duration = event.get('duration')
                    if isinstance(duration, timedelta) and start_time:
                        duration_minutes = duration.total_seconds() / 60
                        start_time_truncated = start_time.replace(minute=0, second=0, microsecond=0)
                        if app not in app_activity_periods:
                            app_activity_periods[app] = []
                        app_activity_periods[app].append({
                            'title': title,
                            'start_time': start_time_truncated.strftime('%Y%m%d %H:%M:%S'),
                            'duration': duration_minutes
                        })

    return app_activity_periods, afk_periods, web_activity


def prepare_data(app_activity_periods, hostname, ip_address):
    """Prepare the data dictionary for JSON structure."""
    data = {
        'hostname': hostname,
        'ip_address': ip_address
    }
    n = 0
    for program, events in app_activity_periods.items():
        duration_total = 0.0
        last_start_time = None
        for event in events:
            if duration_total > 0 and last_start_time != event['start_time']:
                n += 1
                data[f'name_{n}'] = program
                data[f'start_{n}'] = last_start_time
                data[f'duration_{n}'] = round(duration_total, 2)
                duration_total = 0.0
            last_start_time = event['start_time']
            duration_total += event['duration']

        if duration_total > 0:
            n += 1
            data[f'name_{n}'] = program
            data[f'start_{n}'] = last_start_time
            data[f'duration_{n}'] = round(duration_total, 2)

    data['n'] = n
    return data


def send_data(url, data):
    """Send the collected data to the specified URL."""
    try:
        response = requests.post(url, data=data)
        response.raise_for_status()
        print(f"Data successfully sent to {url}. Response status: {response.status_code}")
    except requests.exceptions.RequestException as e:
        print(f"Failed to send data to {url}: {e}")


def main():
    # Initialize ActivityWatch client
    client = initialize_aw_client("my-client")

    # Define the time range
    start_time = get_time_range(minutes=60)

    # Retrieve and filter buckets
    try:
        buckets = client.get_buckets()
    except Exception as e:
        print(f"Error retrieving buckets: {e}")
        sys.exit(1)

    window_buckets, afk_buckets, web_buckets = filter_buckets(buckets)

    # Get events from the filtered buckets
    bucket_event_map, all_events = get_bucket_events(client, window_buckets + afk_buckets + web_buckets, start_time)

    # Process events
    app_activity_periods, afk_periods, web_activity = process_events(bucket_event_map, window_buckets, afk_buckets, web_buckets)

    # Get machine info
    hostname, ip_address = get_machine_info()

    # Prepare data for JSON structure
    data = prepare_data(app_activity_periods, hostname, ip_address)

    # Print data for debugging
    print(json.dumps(data, indent=4))

    # Define the endpoint URL
    url = 'https://panel.qinaya.co/api/mobile.asp?action=monitor'

    # Send data
    send_data(url, data)


if __name__ == '__main__':
    main()

